#========================================================
#
# DMXIS CLIPBOARD PASTE by Kalle Paakkonen - 4 Sep 2018
#                      ENJOY!
#
# Paste the channel values from clipboard to selected
# position or selected channels
#
# If multiple channels are selected, values will be
# pasted by looping the clipboard until all selected
# channels are edited.
#
# If single channel is selected, values will be pasted as
# they were originally starting from the selected channel.
#========================================================

#========================================================
# Edit the following file path to match with CopyChannel.py
#========================================================
clipboardfile = "/tmp/dmxis_clipboard"


import marshal
import os.path
ns = GetNumSelCh()

if not os.path.isfile(clipboardfile):
    Message("Copy something first!")
    raise ValueError("No Channels Copied")

if not ns:
    Message("Select a channel first!")
    raise ValueError("No Channels Selected")

file = None
try:
    file = open(clipboardfile, 'r')
except Exception as e:
    Message("Could not read the clipboard file. Check the macro configuration.")
    raise e

data = file.read();
objList = marshal.loads(data)
file.close()

targetChs = []

# Deselect channels. Pasted Channels will be selected later
# in order to highlight the changed channels
for j in range(ns):
    ch = GetSelCh(j)
    targetChs.append(ch)
    SelectCh(ch, 0)

if len(objList) > 0:
    j = 0
    pasted = 0
    # resolve the amount of channels to edit
    max = len(objList)
    if (len(targetChs) > 1):
        max = len(targetChs)

    origStart = objList[0][0]
    while pasted < max:
        # by default paste items as they were copied
        # starting from the selected channel
        ch = targetChs[0] + objList[j][0] - origStart
        # If multiple channels were selected,
        # paste to those until each selected channels are edited
        if (len(targetChs) > 1):
            ch = targetChs[pasted]

        # Overflow check
        if (ch > 511):
            Message("Not enough channels available!")
            break
            
        SetChVal(ch, objList[j][1])
        #st
        SetStLevel(ch, objList[j][2])
        SetStBand(ch, objList[j][3])
        SetStAttack(ch, objList[j][4])
        SetStRelease(ch, objList[j][5])
        SetStDir(ch, objList[j][6])
        #osc
        SetOscType(ch, objList[j][7])
        SetOscAmount(ch, objList[j][8])
        SetOscChase(ch, objList[j][9])
        SetOscSpeed(ch, objList[j][10])
        SetOscShape(ch, objList[j][11])
        #general
        SetChInvert(ch, objList[j][12])
        SetChEnabled(ch, objList[j][13])
        SelectCh(ch, 1)
        j += 1
        pasted += 1

        # if all copied channels have been pasted, but there is still
        # target channels left, start again from the beginning
        if (j == len(objList)):
            j = 0

    print("Pasted to " + str(pasted) + " channels")
